
local StoneGateV6 = ...

-- Apply building asset processor
StoneGateV6:registerAssetProcessor("models/basic/basicAngle45.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Angle 46° prefab nodes
StoneGateV6:registerAssetId("models/basic/basicAngle45.fbx/Prefab/angle45Tower/step_3_A45_TopPart", StoneGateSetPrefix .. "_A45_TOP_PREFAB")

--------------------------------------------------------------------------
-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGateV6:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		Mover = { DataType = "BUILDING_PART_MOVER" },
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 1 }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	        }
	})
end

local defaultNodePrefixList = {
	StoneGateSetPrefix .. "_A45_TOP",
	StoneGateSetPrefix .. "_A45_TILING1",
	StoneGateSetPrefix .. "_A45_TILING2",
	StoneGateSetPrefix .. "_A45_TILING3",
	StoneGateSetPrefix .. "_A45_TILING1P",
	StoneGateSetPrefix .. "_A45_TILING2P",
	StoneGateSetPrefix .. "_A45_TILING3P"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
end

function registerAngle45(_option)
    -- Register LargeGate prefab nodes
    StoneGateV6:registerAssetId("models/basic/basicAngle45.fbx/Prefab/angle45Tower/step_0_A45_Root" .. _option .. "Part", StoneGateSetPrefix .. "_A45_ROOT" .. _option .. "_PREFAB")
    StoneGateV6:registerAssetId("models/basic/basicAngle45.fbx/Prefab/angle45Tower/step_1_A45_Tiling1" .. _option .. "Part", StoneGateSetPrefix .. "_A45_TILING1" .. _option .. "_PREFAB")
    StoneGateV6:registerAssetId("models/basic/basicAngle45.fbx/Prefab/angle45Tower/step_2_A45_Tiling2" .. _option .. "Part", StoneGateSetPrefix .. "_A45_TILING2" .. _option .. "_PREFAB")
    StoneGateV6:registerAssetId("models/basic/basicAngle45.fbx/Prefab/angle45Tower/step_2_A45_Tiling3" .. _option .. "Part", StoneGateSetPrefix .. "_A45_TILING3" .. _option .. "_PREFAB")

    -- Register the stone gate set A45 tower root part
    StoneGateV6:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_A45_ROOT" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_A45_ROOT" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_A45_ROOT" .. _option .. "_DESC",
	Category = "CORE",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_A45_ROOT" .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_A45_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_A45_TILING3" .. _option .. "_PART",
			StoneGateSetPrefix .. "_A45_TILING2" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_A45_TILING1" .. _option .. "_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true,
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 1 }
		}
	}
    })

    -- Register grass clearing components
    StoneGateV6:registerPrefabComponent("models/basic/basicAngle45.fbx/Prefab/angle45Tower/step_0_A45_Root" .. _option .. "Part", {
	DataType = "COMP_DIRT_CIRCLE",
	HardRadius = 4,
        SoftRadius = 4,
        IsPermanent = true
    })

    -- Override building part components properties (generated by the building asset processor) to add building zones
    StoneGateV6:registerPrefabComponent("models/basic/basicAngle45.fbx/Prefab/angle45Tower/step_0_A45_Root" .. _option .. "Part", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZoneData = {
            DataType = "BUILDING_ZONE", 
            Polygon = polygon.createCircle( 1, { 0, 0 }, 5 ) -- to be recompute
        }
    })
end

registerAngle45("") -- with stone ground
registerAngle45("P") -- with wooden ground

--------------------------------------------------------------------------
-- Register flag assigner component on the collider's type to 'PLATFORM'
--StoneGateV6:configurePrefabFlagList("models/basic/basicAngle45.fbx/Prefab/angle45Tower/step_3_A45_TopPart", { "PLATFORM" })
 
--[[
function registerRoofPart(_nameFbx, _nameAsset)
    StoneGateV6:registerAssetId("models/basic/basicAngle45.fbx/Prefab/angle45Tower/" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration roof part
    StoneGateV6:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 5 }
		}
	}
    })

    StoneGateV6:registerPrefabComponent("models/basic/basicAngle45.fbx/Prefab/angle45Tower/" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the roofs)
	BuildingZone = { 0, 0 } -- For future snapping over large gate
    })
end

registerRoofPart( "A45Roof", "A45_ROOF" )
]]
---- Ramparts...

-- Apply building asset processor
StoneGateV6:registerAssetProcessor("models/ramparts/rampartAngle45.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

function registerRampartPart(_nameFbx, _nameAsset)
    StoneGateV6:registerAssetId("models/ramparts/rampartAngle45.fbx/Prefab/angle45Tower/step_4_A45_" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_A45_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration rampart part
    StoneGateV6:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_A45_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_A45_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_A45_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_A45_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 1 }
		}
	}
    })

    StoneGateV6:registerPrefabComponent("models/ramparts/rampartAngle45.fbx/Prefab/angle45Tower/step_4_A45_" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the roofs)
	BuildingZone = { 1, 1 }
    })
end

registerRampartPart( "RampartA", "RAMPART_A" )
